@@ -921,6 +921,18 @@ e_mail_notes_editor_init (EMailNotesEditor *notes_editor)
 {
 }
 
+static gboolean
+set_preformatted_block_format_on_idle_cb (gpointer user_data)
+{
+	EContentEditor *cnt_editor = user_data;
+
+	g_return_val_if_fail (E_IS_CONTENT_EDITOR (cnt_editor), FALSE);
+
+	e_content_editor_set_block_format (cnt_editor, E_CONTENT_EDITOR_BLOCK_FORMAT_PRE);
+
+	return FALSE;
+}
+
 static EMailNotesEditor *
 e_mail_notes_editor_new_with_editor (EHTMLEditor *html_editor,
 				     GtkWindow *parent,
@@ -1075,8 +1087,12 @@ e_mail_notes_editor_new_with_editor (EHTMLEditor *html_editor,
 	gtk_widget_grab_focus (GTK_WIDGET (cnt_editor));
 
 	settings = e_util_ref_settings ("org.gnome.evolution.mail");
-	e_content_editor_set_html_mode (
-		cnt_editor, g_settings_get_boolean (settings, "composer-send-html"));
+	e_content_editor_set_html_mode (cnt_editor, g_settings_get_boolean (settings, "composer-send-html"));
+	if (!g_settings_get_boolean (settings, "composer-send-html") &&
+	    g_settings_get_boolean (settings, "composer-plain-text-starts-preformatted")) {
+		g_idle_add_full (G_PRIORITY_LOW, set_preformatted_block_format_on_idle_cb,
+			g_object_ref (cnt_editor), g_object_unref);
+	}
 	g_object_unref (settings);
 
 	g_signal_connect (
